@@ -1,6 +1,6 @@
 package Workflow::Action;
 
-# $Id: Action.pm 539 2012-10-25 16:19:14Z jonasbn $
+# $Id: Action.pm 542 2012-10-25 16:39:01Z jonasbn $
 
 # Note: we may implement a separate event mechanism so that actions
 # can trigger other code (use 'Class::Observable'? read observations
@@ -18,7 +18,8 @@ use Carp qw(croak);
 $Workflow::Action::VERSION = '1.10';
 
 my @PROPS = qw( name class description );
-__PACKAGE__->mk_accessors(@PROPS);
+my @INTERNAL = qw( _factory );
+__PACKAGE__->mk_accessors(@PROPS, @INTERNAL);
 
 ####################
 # INPUT FIELDS
@@ -50,7 +51,7 @@ sub add_validators {
     my ( $self, @validator_info ) = @_;
     my @validators = ();
     foreach my $conf (@validator_info) {
-        my $validator = FACTORY->get_validator( $conf->{name} );
+        my $validator = $self->_factory()->get_validator( $conf->{name} );
         my @args      = $self->normalize_array( $conf->{arg} );
         push @validators,
             {
@@ -106,6 +107,7 @@ sub init {
     # So we don't destroy the original...
     my %copy_params = %{$params};
 
+    $self->_factory( $wf->_factory() );
     $self->class( $copy_params{class} );
     $self->name( $copy_params{name} );
     $self->description( $copy_params{description} );
