@@ -1,6 +1,6 @@
 package Workflow::Factory;
 
-# $Id: Factory.pm 539 2012-10-25 16:19:14Z jonasbn $
+# $Id: Factory.pm 552 2012-10-26 18:35:11Z jonasbn $
 
 use warnings;
 use strict;
@@ -10,7 +10,6 @@ use Log::Log4perl qw( get_logger );
 use Workflow::Exception qw( configuration_error workflow_error );
 use Carp qw(croak);
 use English qw( -no_match_vars );
-
 $Workflow::Factory::VERSION = '1.20';
 
 my ($log);
@@ -252,11 +251,10 @@ sub _add_workflow_config {
         # When we instantiate a new workflow we pass these objects
 
         foreach my $state_conf ( @{ $workflow_config->{state} } ) {
-
             # Add the workflow type to the state conf.
-            $state_conf->{type} = $wf_type;
-
-            my $wf_state = Workflow::State->new($state_conf);
+            $state_conf->{type} = $wf_type;                        
+            my $wf_state = Workflow::State->new($state_conf, $self);
+            
             push @{ $self->{_workflow_state}{$wf_type} }, $wf_state;
         }
 
@@ -326,18 +324,20 @@ sub _load_class {
 }
 
 sub create_workflow {
-    my ( $self, $wf_type, $context ) = @_;
+    my ( $self, $wf_type, $context, $wf_class ) = @_;
     $log ||= get_logger();
 
+    $wf_class = 'Workflow' unless($wf_class);
+    
     my $wf_config = $self->_get_workflow_config($wf_type);
     unless ($wf_config) {
         workflow_error "No workflow of type '$wf_type' available";
     }
 
     my $wf
-        = Workflow->new( undef,
+        = $wf_class->new( undef,
         $wf_config->{initial_state} || $DEFAULT_INITIAL_STATE,
-        $wf_config, $self->{_workflow_state}{$wf_type} );
+        $wf_config, $self->{_workflow_state}{$wf_type}, $self );
     $wf->context( $context || Workflow::Context->new );
     $wf->last_update( DateTime->now( time_zone => $wf->time_zone() ) );
     $log->is_info
@@ -379,9 +379,11 @@ sub create_workflow {
 }
 
 sub fetch_workflow {
-    my ( $self, $wf_type, $wf_id, $context ) = @_;
+    my ( $self, $wf_type, $wf_id, $context, $wf_class ) = @_;
     $log ||= get_logger();
 
+    $wf_class = 'Workflow' unless($wf_class);
+    
     my $wf_config = $self->_get_workflow_config($wf_type);
     unless ($wf_config) {
         workflow_error "No workflow of type '$wf_type' available";
@@ -397,7 +399,8 @@ sub fetch_workflow {
         "[Last update: $wf_info->{last_update}]"
         );
     my $wf = Workflow->new( $wf_id, $wf_info->{state}, $wf_config,
-        $self->{_workflow_state}{$wf_type} );
+        $self->{_workflow_state}{$wf_type}, $self );
+     
     $wf->context( $context || Workflow::Context->new )
         if ( not $wf->context() );
     $wf->last_update( $wf_info->{last_update} );
@@ -850,12 +853,19 @@ The factory is a singleton, this is how you get access to the
 instance. You can also just import the 'FACTORY' constant as in the
 L</SYNOPSIS>.
 
-=head3 create_workflow( $workflow_type )
+=head3 create_workflow( $workflow_type, $context, $wf_class )
 
 Create a new workflow of type C<$workflow_type>. This will create a
 new record in whatever persistence mechanism you have associated with
 C<$workflow_type> and set the workflow to its initial state.
 
+The C<$context> argument is optional, you can pass an exisiting instance 
+of Workflow::Context to be reused. Otherwise a new instance is created.
+
+The C<$wf_class> argument is optional. Pass it the name of a class to be
+used for the workflow to be created. By default, all workflows are of the 
+I<Workflow> class.
+
 Any observers you've associated with this workflow type will be
 attached to the returned workflow object.
 
@@ -864,13 +874,20 @@ C<WORKFLOWS ARE OBSERVABLE> in L<Workflow> for more.
 
 Returns: newly created workflow object.
 
-=head3 fetch_workflow( $workflow_type, $workflow_id )
+=head3 fetch_workflow( $workflow_type, $workflow_id, $context, $wf_class )
 
 Retrieve a workflow object of type C<$workflow_type> and ID
 C<$workflow_id>. (The C<$workflow_type> is necessary so we can fetch
 the workflow using the correct persister.) If a workflow with ID
 C<$workflow_id> is not found C<undef> is returned.
 
+The C<$context> argument is optional, you can pass an exisiting instance 
+of Workflow::Context to be reused. Otherwise a new instance is created.
+
+The C<$wf_class> argument is optional. Pass it the name of a class to be
+used for the workflow to be created. By default, all workflows are of the 
+I<Workflow> class.
+
 Any observers you've associated with this workflow type will be
 attached to the returned workflow object.
 
